definition module Data.Functor

class Functor f
where
	/**
	 * Applies a function to a functor.
	 *
	 * The functor argument is lazy to avoid a cycle in spine in certain cases.
	 * The type of most instances can be defined with the `f a` argument
	 * strict.
	 */
	fmap :: (a -> b) (f a) -> f b

	//* Infix version of `fmap`.
	(<$>) infixl 4 :: (a -> b) (f a) -> f b | Functor f
	(<$>) f fa :== fmap f fa

	//* Flipped version of `<$>`.
	(<$&>) infixl 1 :: (f a) (a -> b) -> f b | Functor f
	(<$&>) fa f :== fmap f fa

	(<$) infixl 4 :: a (f b) -> f a | Functor f
	(<$) x fa :== fmap (\_ -> x) fa

	($>) infixl 4 :: (f b) a -> f a | Functor f
	($>) fa x :== x <$ fa

	void :: (f a) -> f () | Functor f
	void x :== () <$ x
